import json
import os
from pathlib import Path

from bson import ObjectId
from pymongo import MongoClient
from datetime import datetime

from config.gitUtils import GitUtils
from config.utils import Utils


class EncoursDao:
    client = MongoClient('localhost', 27017)
    chemin = "/home/OpenQuizz/en_cours/"
    gitUtils = GitUtils()
    utils = Utils()

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Fonction qui permet de savoir si un contenu en cours à fait l'objet d'une demande de publication
    # @param idEnCours l'id du contenu en cours
    # @param l'auteur l'auteur potentiel de la demande de publication
    def is_en_cours_demande(self, idEnCours, auteur):
        return self.base_donnees.demandesPub.find({"id": idEnCours} and {"auteurDemande": auteur}).count() != 0

    ## récupérer l'auteur initial d'un contenu en cours (peut être l'auteur effectuant la modification ou non)
    # @param idContenuEnCours l'id du contenu en cours de modification
    # @return l'auteur intial du contenu en cours
    def get_auteur_origine_contenu_en_cours(self, id_contenu_en_cours):
        id_contenu = self.base_donnees.encours.find_one({"_id": ObjectId(id_contenu_en_cours)})["reference"]
        return self.base_donnees.contenu.find_one({"_id": id_contenu})["auteur"]

    ## récupérer l'id initial d'un contenu en cours (peut être l'auteur effectuant la modification ou non)
    # @param idContenuEnCours l'id du contenu en cours de modification
    # @return l'id intial du contenu en cours
    def get_id_origine_contenu_en_cours(self, id_contenu_en_cours):

        if type(id_contenu_en_cours) == str:
            id_contenu_en_cours = ObjectId(id_contenu_en_cours)

        return self.base_donnees.encours.find_one({"_id": id_contenu_en_cours})["reference"]

    ## permet de récupérer tous les contenus en cours pour un auteur
    # @param auteur le nom de l'auteur
    # @return les contenus en cours de l'auteur
    def get_contenus_en_cours_auteur(self, auteur):
        return self.base_donnees.encours.find({"auteur": auteur})

    ## récupérer un contenu en cours
    # @param idContenuEnCours l'id du contenu en cours de modification
    # @return le contenu en cours
    def get_contenu_en_cours_via_id(self, id_contenu_en_cours):
        # id_contenu = Utils.getIdContenuAssocieAUnEnCours(idContenuEnCours)
        if type(id_contenu_en_cours) == str:
            id_contenu_en_cours = ObjectId(id_contenu_en_cours)

        return self.base_donnees.encours.find_one({"_id": id_contenu_en_cours})

    ## supprimer un contenu en cours avec l'id
    # @param idContenuEnCours id du contenu a supprimer
    def supprimer_contenu_en_cours_via_id(self, id_contenu_en_cours):

        # on supprime les fichiers
        chemin = self.get_chemin_contenu_en_cours(id_contenu_en_cours)
        os.remove(chemin)
        # puis les métadonnées
        self.base_donnees.encours.delete_one({"_id": ObjectId(id_contenu_en_cours)})

    ## création d'un contenu en cours
    # @param auteur -> l'auteur du contenu en cours.
    # @param contenu -> le contenu d'origine
    # @param version la version du contenu d'origine
    # @param provenance pour déterminer si le contenu d'origine est un tiers ou un perso
    def creer_en_cours(self, auteur, provenance, contenu, titre):
        from modele.contenu_dao import ContenuDao

        if contenu["versionEnCours"]["nbreNotes"] == 0:
            moyenne = 0
        else:
            moyenne = contenu["versionEnCours"]["note"] / contenu["versionEnCours"]["nbreNotes"]

        # print("auteur : ", auteur)
        # print("provenance : ", provenance)
        # print("titre : ", titre)
        # print("id : ", contenu["_id"]["$oid"])
        # print("type id : ", type(contenu["_id"]["$oid"]))
        # print("version : ", contenu["versionEnCours"]["numero"])



        if ContenuDao().is_anonyme(contenu["_id"]["$oid"]):

            self.base_donnees.encours.insert_one(
                {"date": datetime.now().strftime("%d/%m/%y"), "fichier": "", "auteur": auteur,
                 "reference": ObjectId(contenu["_id"]["$oid"]), "version": contenu["versionEnCours"]["numero"],
                 "origine": provenance, "tags_fixes": json.loads(contenu["tags_fixes"]),
                 "titre": titre, "type": contenu["type"], "source": contenu["source"], "moyenne": moyenne,
                 "anonyme": "oui"})

        else:

            self.base_donnees.encours.insert_one(
                {"date": datetime.now().strftime("%d/%m/%y"), "fichier": "", "auteur": auteur,
                 "reference": ObjectId(contenu["_id"]["$oid"]), "version": contenu["versionEnCours"]["numero"],
                 "origine": provenance, "tags_fixes": json.loads(contenu["tags_fixes"]),
                 "titre": titre, "type": contenu["type"], "source": contenu["source"], "moyenne": moyenne})
        id = self.recuperer_dernier_id()
        chemin_tags = '/'.join(json.loads(contenu["tags_fixes"])) + "/"

        nom_fichier = (contenu["fichier"][contenu["fichier"].rfind("/") + 1:])

        chemin_fichier = self.chemin + chemin_tags + nom_fichier + "/"
        Path(chemin_fichier).mkdir(0o777, parents=True, exist_ok=True)

        myFile = open(chemin_fichier + nom_fichier, "w")
        # y ajouter le texte du contenu
        myFile.write(self.utils.recuperer_contenu_fichier(contenu["_id"]["$oid"]))
        myFile.close()

        # mettre a jour le chemin du contenu
        self.base_donnees.encours.update_one({"_id": ObjectId(id)},
                                             {"$set": {"fichier": chemin_tags + nom_fichier + "/" + nom_fichier}})

        # si le contenu de base possède un descriptif, on le vreporte ici
        if "descriptif" in contenu:
            self.base_donnees.encours.update_one({"_id": ObjectId(id)},
                                                 {"$set": {"descriptif": contenu["descriptif"]}})

    ## Modification d'un titre d'un contenu en cours
    # @param id_en_cours l'id du contenu en cours
    # @param nouveau_titre le titre à changer
    def changer_titre(self, id_en_cours, nouveau_titre):
        # print("Je suis ici : " + nouveau_titre)
        self.base_donnees.encours.update_one({"_id": ObjectId(id_en_cours)}, {"$set": {"titre": nouveau_titre}})

    ## Récupération d'un titre d'un contenu en cours
    # @param id_en_cours l'id du contenu en cours
    def get_titre_en_cours(self, id_en_cours):
        return self.base_donnees.encours.find_one({"_id": ObjectId(id_en_cours)})["titre"]

    ## création d'un contenu en cours, dans l'état refuse
    # @param version la version du contenu d'origine
    # @param nomBranche -> plus utile.
    def creer_en_cours_refuse(self, id_contenu_en_cours):
        self.base_donnees.encours.update_one({"_id": ObjectId(id_contenu_en_cours)}, {"$set": {"refuse": "oui"}})
        self.base_donnees.encours.update_one({"_id": ObjectId(id_contenu_en_cours)}, {"$unset": {"isPublie": "oui"}})

    ## récupérer le dernier id inséré dans encours
    # @return l'id de la derniere insertion dans encours.
    def recuperer_dernier_id(self):
        dernier_id_insere = self.base_donnees.encours.find().sort("_id", -1).limit(1)

        record = next(dernier_id_insere, None)
        return str(record.get("_id"))

    ## récupérer le chemin associé au contenu en cours
    # @param id_contenu le contenu en cours concerné
    # @return le chemin associé à id_contenu.
    def get_chemin_contenu_en_cours(self, id_contenu):
        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)

        return self.chemin + self.base_donnees.encours.find_one({"_id": id_contenu})["fichier"]
